package com.pitong.business;


import com.pitong.business.item.DataPackage;
import com.pitong.business.item.Picture;
import com.pitong.business.service.IPService;
import com.pitong.business.service.ServerService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.*;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.builder.SpringApplicationBuilder;
import org.springframework.stereotype.Service;

import java.io.*;
import java.lang.reflect.Method;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.LinkedList;
import java.util.List;
import java.util.Stack;

@Service
@SpringBootApplication
public class Application implements ApplicationRunner {

    @Autowired
    private ServerService serverService;
    @Autowired
    private IPService ipService;
    @Value("${slot}")
    private Long slot;  //时隙值

    public static void main(String[] args) {
        new SpringApplicationBuilder(Application.class)
                .web(WebApplicationType.NONE) // .REACTIVE, .SERVLET
                .bannerMode(Banner.Mode.OFF)
                .run(args);
    }

    @Override
    public void run(ApplicationArguments args) throws Exception {
        ServerSocket serverSocket = null;
        InputStream inputStream = null;
        ObjectInputStream objectInputStream = null;
        Long startime = System.currentTimeMillis();
        try {
            serverSocket = new ServerSocket(2020);
            while (true) {
                Socket client = serverSocket.accept();
                inputStream = client.getInputStream();
                objectInputStream = new ObjectInputStream(new BufferedInputStream(inputStream));
                Object object = objectInputStream.readObject();
                DataPackage dataPackage = (DataPackage) object;

                Stack[] route = dataPackage.getRoute();
                int routeNumber = dataPackage.getRouteNumber();
                int[] mapping = {0,1,2,3,4,5,5,6};
                List<Integer> sequencet = dataPackage.getSequence();
                int currentHop = dataPackage.getCurrentHop();
                if (!route[routeNumber].empty()){
                    //发送到其他卫星计算，本卫星只转发
                    //get下一转发节点
                    Integer nextHop = (Integer) route[routeNumber].pop();
                    System.out.println("cover transfer next sate:"+nextHop);
                    //重新组合数据包
                    dataPackage.setRoute(route);
                    dataPackage.setCurrentHop(nextHop);
                    transferPackage(dataPackage, nextHop, startime);
                    continue;
                }else if(sequencet.get(mapping[routeNumber])!=currentHop){

                    int nextHop = sequencet.get(mapping[routeNumber]);
                    System.out.println("routeNumber"+routeNumber);
                    System.out.println("currentHop:"+currentHop+"  task exec sate"+nextHop);
                    dataPackage.setCurrentHop(nextHop);
                    transferPackage(dataPackage, nextHop, startime);
                    continue;
                }


                int number = dataPackage.getNumber();
                List<Integer> sequence = dataPackage.getSequence();
                Picture picture = dataPackage.getPicture();
                String loc = dataPackage.getLoc();
                double eta = dataPackage.getEta();

                Long Currentime = dataPackage.getCurrentime();
                Long fun1time = dataPackage.getFun1time();
                List<Double> compCapacity = dataPackage.getCompCapacity();
                LinkedList<Double> tranDelay =dataPackage.getTranDelay();
                System.out.println("route array"+tranDelay);
                System.out.println("=====================");
                System.out.println("running");

                String methodName = "func" + number;
                Method method = serverService.getClass().getMethod(methodName, Class.forName("java.util.List"), Picture.class, Class.forName("java.lang.String"), Class.forName("java.lang.Double"),
                        Class.forName("java.lang.Long"),Class.forName("java.lang.Long"), Class.forName("java.lang.Long"), Class.forName("java.util.List"), route.getClass(), Class.forName("java.util.LinkedList"));
                method.invoke(serverService, sequence, picture, loc, eta, Currentime, startime, fun1time, compCapacity, route, tranDelay);
            }
        } catch (IOException | ClassNotFoundException e) {
            e.printStackTrace();
        } finally {
            try {
                if (objectInputStream != null) {
                    objectInputStream.close();
                }
                if (inputStream != null) {
                    inputStream.close();
                }
                if (serverSocket != null) {
                    serverSocket.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    private void transferPackage(DataPackage dataPackage, int nextHop, long startime) {
        //获取目标节点ip
        String app_name = "sate" + nextHop;
        String IP = ipService.getIpByAppName(app_name);
        //发送至目标
        send(dataPackage, IP, 2020, startime);
    }
    private void send(DataPackage dataPackage, String ip, int port, Long startime) {
        Socket socket = null;
        try {
            socket = new Socket(ip, port);
            ObjectOutputStream objectOutputStream = new ObjectOutputStream(socket.getOutputStream());
            objectOutputStream.writeObject(dataPackage);
        } catch (IOException e) {
            Long waitime = slot-(System.currentTimeMillis()-startime)%slot;
            try {
                Thread.sleep(waitime+10);
            } catch (InterruptedException ex) {
                ex.printStackTrace();
            }
            System.out.println(">>>>>>>>>>>>>>>>>>>");
            System.out.println("blocked current slot: " + ((System.currentTimeMillis()-startime)/slot + 1));
            System.out.println("try again ...");
            send(dataPackage, ip, port, startime);
        } finally {
            try {
                if (socket != null) {
                    socket.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
}
