package top.ninwoo.cloud.service.bpso;

import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import top.ninwoo.cloud.config.BpsoConfig;
import top.ninwoo.cloud.service.csv.Wteg;

import java.text.ParseException;
import java.util.LinkedList;
import java.util.Random;
import java.util.Stack;

/**
 * 求某一数据量下的最佳映射结果和业务处理时延
 * 输出最佳粒子的位置和适应度值
 * 目前只允许计算过程跨单时隙
 * @jthu
 */
@Component
@Slf4j
public class Bpso {


    //最大权值
    public static final int MAXWEIGHT = 10000*1000;

    /*
     * 业务参数
     */
    //数据量MB？
//    static double[] D = {10, 10, 1, 3, 0.8, 2.4, 0.7, 1.2};
    static double[] D;
    //子任务数
//    static int c = 8; //8
    static int c;
    //子任务依赖关系 边
    static int[][] link; //
    //task编号
    static int[] func;


    //固定映射任务数==起始任务+终止任务
    static final  int C_STABLE = 2;

/*    //实际需映射任务数
    static int n1 = c - C_STABLE;*/

    /*
     * 网络参数
     */
    //设备数
//    public static int num = 12;
    public static int num;

    //设备计算能力参数s  GB/s
//    static double[] nt = {1.112341341313, 0.5196272109574349, 0.675827555541419, 0.7637869915859579, 0.7998320296858645,
//            0.638652700648973, 0.6396760676518485, 0.5615337810185533, 0.6356797503522186, 0.8266103444996542,0.8102279249524735, 1.17057296148341216};  //32节点
    static double[] nt;
    //时隙大小
    public static double slot = 100*1000;
    //时隙数目
    static int slotnum;
    //网络节点数
    public static int dev;
    //初始化时延偏移量
    //这里需要一个时钟输入
    static double testLatency = 0;

    //固定映射设备数==起始设备+终止设备
    static final  int N_STABLE = 2;
    //实际可映射设备数
    static int n2;
/*    //单时隙可映射设备数
    static int mapdev = num - N_STABLE ;*/
    /*
     * BPSO参数
     */
    //粒子群规模
    static int n;
    //种群迭代次数
    static int k;
    //惯性权重，加速因子1，2
    static double s = 1.5; static int c1 = 1; static int c2 = 1;

    public static void main(String[] args) throws ParseException {
        //子任务依赖关系 边
        int[][] link = {{0, 1, 1, 2, 3, 4, 5, 6}, {1, 2, 3, 4, 5, 6, 6, 7}};
        //task编号
        int[] func = {0, 2, 3, 3, 1, 1, 4, 0};
        Bpso doMap = new Bpso();
//        doMap.doMapApp(link,func);
    }


    //节点最终分配结果
    public static int[] result;
    //节点间最短路由
    public static Stack[] route;
    //业务发起时刻
    public static long startT;
    //传输时延约束
    public static LinkedList<Double> list = new LinkedList<>();
    //路由表
    public static int[][] lastHop;

    public void doMapApp(int[][] link1, int[] func1,BpsoConfig bpsoConfig) throws ParseException {

        // 加载粒子群配置项 种群 迭代 任务 设备等参数
        parseConfig(bpsoConfig);
        //实际需映射任务数
        int n1 = c - C_STABLE;
        //单时隙可映射设备数
        int mapdev = num - N_STABLE ;
        //节点最终分配结果
        result = new int[c];
        link = link1;
        func = func1;
        route = new Stack[link[0].length+1];


        for (slotnum = 1;slotnum<=3;slotnum++){
            //slotnum = 3;
            dev = num*slotnum;
            n2 = dev - N_STABLE*slotnum;
            //1组装最短路矩阵  加权时间扩展图
//            double[][] edgeWeight = Wteg.getWteg(num*slotnum);
            double[][] edgeWeight = Wteg.getIridiumWteg(num*slotnum);

            InterSlotDijstraV2 dijs = new InterSlotDijstraV2(num, slot);
            InterSlotDijstraV2.pathRecord pathRecord = dijs.dijstra(edgeWeight, testLatency);
            //2最短路由矩阵  时间扩展图最短路由时延矩阵
            double[][] lt = pathRecord.pathWeight;
            double maxLatency = 0;
            for (int j = 0;j<lt.length;j++) {
                for (int i = 0; i < lt.length; i++) {
                    if (lt[j][i]!=MAXWEIGHT && lt[j][i]>maxLatency){
                        maxLatency = lt[j][i];
                    }
                }
            }
            //将不可达路由时延设置为较高时延
            for (int j = 0;j<lt.length;j++) {
                for (int i = 0; i < lt.length; i++) {
                    if (lt[j][i]==MAXWEIGHT){
                        lt[j][i] =  maxLatency + 500;
                    }
                }
            }
            lastHop = pathRecord.lastHop;
//            原本求的是最后一跳路由，但是现在求得是下一跳路由
            for (int j = 0;j<lastHop.length;j++){
                for(int i = 0;i<lastHop.length;i++){
                    int tmp0 = lastHop[j][i];
                    while (tmp0 != 0){
                        lastHop[j][i] = tmp0;
                        tmp0 = lastHop[j][tmp0];
                    }
                }
            }
            //将不可达路由连接，隐藏网络不连通性，搞成环了
            for (int j = 0;j<lastHop.length;j++){
                for(int i = 0;i<lastHop.length;i++){
                    if (lastHop[j][i] == 0) {
                        lastHop[j][i] = i;
                    }
                }
            }
            // 一跳路由转发表
            String[][] routetest = new String[lastHop.length][lastHop[0].length];
            for (int l = 0; l < lastHop.length/num; l++) {
                for (int i = l*num; i < num+l*num; i++) {
                    for (int j = l*num; j < lastHop.length; j++) {
                        routetest[i][j] = "sate" + (lastHop[i][j]);
                        //到自己
                        if((i != 0 || i != num*l)&(routetest[i][j].equals("sate0"))){
                            routetest[i][j] = "sate"+i;
                        }
                        //表示不可达
                        if(j==num*l){
                            routetest[i][j] = null;
                        }
                    }
                }
            }

            //执行粒子群算法，得路径计算结果
            Bpso doMap = new Bpso();
            Par best_par = doMap.main_bpso(k, c, dev, n1, n2, D, lt, nt, link, n, s, c1, c2, num, slot, mapdev);
            if ((1/best_par.fit) != 0){
//                log.info("规划最短时延："+(1/best_par.fit));
                //赋值节点映射结果
                int itemp = 0;
                for (int a:best_par.taskfinal
                ) {
                    result[itemp++] = (a)+1;
                }
                //输出最短路由数组
                for (int i =0;i<result.length-1;i++){
                    list.add(lt[result[i]-1][result[i+1]-1]);
                }
                //赋值最短路由
                //引用类型数组的二次声明
                for (int i = 0;i<route.length;i++){
                    route[i] = new Stack<Integer>();
                }
                //3根据分配结果，回溯每一条路由
                lastHop = pathRecord.lastHop;
                //输出最短路由[1, 2, 3, 3, 9, 16, 40]
                for (int i = 0;i<route.length-1;i++){
                    if (route[i+1].empty()){}

//                            System.out.print(result[link[0][i]]+"-->"+result[link[1][i]]);
                    else {
//                            System.out.print(result[link[0][i]] + "-->");
                        Stack<Integer> stack = new Stack<>();
                        while (!route[i+1].isEmpty()){
                            int pop = (int) route[i+1].pop();
//                                System.out.print(pop + "-->");
                            //顺便将数值转换
                            stack.push((pop-1)%num+1);
                        }
                            /*Enumeration items = route[i].elements(); // 得到 stack 中的枚举对象
                            while (items.hasMoreElements()) //显示枚举（stack ） 中的所有元素
                                System.out.print(items.nextElement() + "-->");*/

//                            System.out.print(result[link[1][i]]);
                        while(!stack.isEmpty()){
                            route[i+1].push(stack.pop());
                        }
                    }
//                        System.out.println(); //换行
                }
                //规范result 和 route 数据
                for (int i = 0;i<result.length;i++){
                    result[i] = ((result[i]-1)%num);
                }
                //System.out.println(Arrays.toString(result));
                break;
            }

        }
    }

    private void parseConfig(BpsoConfig bpsoConfig) {
        k = Integer.parseInt(bpsoConfig.getK());
        n = Integer.parseInt(bpsoConfig.getN());
        c = Integer.parseInt(bpsoConfig.getC());
        num = Integer.parseInt(bpsoConfig.getDev());
        String bpsoConfigD = bpsoConfig.getD();
        String[] bpsoConfigDs = bpsoConfigD.split(",");
        double[] Dc = new double[c];
        for (int i = 0;i<c;i++){
            Dc[i] = Double.parseDouble(bpsoConfigDs[i]);
        }
        D = Dc;
        String[] bpsoConfigDevs = bpsoConfig.getNt().split(",");
        double[] ntdev = new double[num];
        for (int i = 0;i<num;i++){
            ntdev[i] = Double.parseDouble(bpsoConfigDevs[i]);
        }
        nt = ntdev;
    }

    /*
     * 粒子群算法主方法
     */
    private Par main_bpso(int k, int c, int dev, int n1, int n2,
                          double[] D, double[][] lt, double[] nt, int[][] link, int n,double s, int c1, int c2, int num, double slot, int mapdev) {
        //初始化粒子群数组
        int nn = n1*n2;  //粒子维度
        Par best_par = new Par(nn, c);
        Par[] par_array = new Par[n];
        Par [] temp = new Par[2];			//存放返回值

        for (int i = 0; i < par_array.length; i++) {
            par_array[i] = new Par(nn, c);
            initialize(par_array[i],n1,n2, c, dev, num, mapdev);

            //计算粒子适应度值
            par_array[i].fit = compute_fit(par_array[i], D, lt, nt, c, dev, n1, n2, nn, link, num, slot);

            //更新群体最优粒子
//        	Par [] temp = new Par[2];			//存放返回值
            temp = swarmBest(par_array[i], best_par, nn, c);
            par_array[i] = temp[0];
            best_par = temp[1];
        }

        /*
         * 更新粒子群
         */
        for (int i = 0; i < k; i++) {			//迭代次数
            for (int j = 0; j < n; j++) {		//种群规模

                //更新粒子位置、速度
                update_par(par_array[j], best_par, nn, s, c1, c2, c, n1, n2, dev, num, mapdev);

                //计算粒子适应度值
                par_array[j].fit = compute_fit(par_array[j], D, lt, nt, c, dev, n1, n2, nn, link, num, slot);

                //更新群体最优粒子
                temp = swarmBest(par_array[j], best_par,nn, c);
                par_array[j] = temp[0];
                best_par = temp[1];
            }
/*            log.info("当前迭代次数："+i);
            log.info("最佳粒子时延："+(1/best_par.fit));*/

        }
        return best_par;
    }

    /*
     * 初始化粒子位置、速度
     */
    private void initialize(Par par, int n1, int n2, int c, int dev, int num, int mapdev) {
        Random random = new Random();
        boolean flag = true;
        while (flag) {
            for (int i = 0; i < par.x.length; i++) {
                par.x[i] = 0;
            }

            //初始化位置
            for (int i = 0; i < par.x.length; i += n2) {
                int rand = random.nextInt(n2);       //功能：随机生成1 -- n2的任意 int 值
                par.x[i + rand] = 1;
            }

            //extract 提取映射结果
            par.task[0] = 0;
            par.task[c - 1] = dev - 1;
            int index = 1;
            for (int i = 0; i < n1 * n2; i += n2) {
                for (int j = 0; j < n2; j++) {
                    if (par.x[i + j] == 1) {
                        par.task[index] = num * (j / mapdev) + (j % mapdev) + 1;  //
                        index++;
                        break;
                    }
                }
            }
            //去重
            if(isRepitition(par.task)){
                flag = false;
            }
        }

/*        //排序
        int temp;
        for(int p = 0; p<(c-1); p++) {
            for(int j = 0; j <(c-1-p); j++) {
                if(par.task[j] > par.task[j + 1]) {
                    temp = par.task[j + 1];
                    par.task[j + 1] = par.task[j];
                    par.task[j] = temp;
                }
            }
        }*/
        for(int i = 0; i < c; i++) {
            par.taskfinal[i] = par.task[i];
            if(par.taskfinal[i]>dev){
                System.out.println(286);
            }
        }
        //初始化速度
        for (int i = 0; i < par.v.length; i++) {
            double value = random.nextDouble();
            par.v[i] = value;
        }
    }

    /*
     * 更新群体最优粒子
     */
    private Par[] swarmBest(Par par, Par bestPar, int nn, int c) {
        Par[] temp = new Par[2];
        //更新局部最优
        if (par.fit > par.best_fit) {
            par.best_fit = par.fit;
            for (int j = 0; j <  nn; j++) {
                par.bestx[j] = par.x[j];
            }
        }
        temp[0] = par;
        //更新全局最优
        if (par.best_fit > bestPar.fit) {
            bestPar.fit = par.best_fit;
            for (int j = 0; j < nn; j++) {
                bestPar.x[j] = par.bestx[j];
            }
            for(int k =0; k<c; k++) {
                bestPar.task[k] = par.task[k];
                bestPar.taskfinal[k] = par.taskfinal[k];
            }
        }
        temp[1] = bestPar;

        return temp;
    }

    /*
     * 更新粒子位置、速度
     */
    private void update_par(Par par, Par best_par, int nn, double s, int c1, int c2, int c,
                            int n1, int n2, int dev, int num, int mapdev) {
        Random random = new Random();
        /**
         * 公式按公式更新粒子后如果不符合映射规则要求，则再次更新
         * 为此设计布尔变量flag & flag2
         * flag控制是否进行新一轮粒子更新
         * flag2控制是否进行粒子后续映射位置检查
         * 判断流程为 ：初始化flag-->更新粒子一次-->初始化flag2-->判断粒子是否合法-->合法则置flag为false----------------------------
         * 																																		-->非法则置flag2为false,并break内部for循环
         * --------------------------------------------------------------------->完成粒子更新
         * -->并且continue本次外部for循环-->再重新更新粒子一次
         */
//        boolean flag = true;
//        while (flag) {
//        while (true) {

        boolean flag = true;

        while(flag) {
            Par oldpar = par;

            for (int i = 0; i < nn; i++) {
                //更新速度
                oldpar.v[i] = s * par.v[i] + c1 * random.nextDouble() * (best_par.bestx[i] - par.x[i]) + c2 * random.nextDouble() * (par.bestx[i] - par.x[i]);
                //par.v[i] = s * par.v[i] + c1 * random.nextDouble() * (best_par.bestx[i] - par.x[i]) + c2 * random.nextDouble() * (par.bestx[i] - par.x[i]);
                if (oldpar.v[i] > 1) {
                    oldpar.v[i] = 1;
                }
                if (oldpar.v[i] < -1) {
                    oldpar.v[i] = -1;
                }
            }
            //更新位置
            int maxIndex = 0;//假设第一个元素为最大值 那么下标设为0
            for (int i = 0; i < par.v.length - 1; i++) {
                if (oldpar.v[maxIndex] < oldpar.v[i + 1]) {
                    maxIndex = i + 1;
                }
            }
            for (int i = (maxIndex / n2) * n2; i < (maxIndex / n2 + 1) * n2; i++) {
                if (i == maxIndex) {
                    oldpar.x[i] = 1;
                } else {
                    oldpar.x[i] = 0;
                }
            }
            //extract 提取映射结果
            oldpar.task[0] = 0;
            oldpar.task[c - 1] = dev - 1;
            int index = 1;
            for (int i = 0; i < n1 * n2; i += n2) {
                for (int j = 0; j < n2; j++) {
                    if (oldpar.x[i + j] == 1) {
                        oldpar.task[index] = num * (j / mapdev) + (j % mapdev) + 1;
                        index++;
                        break;
                    }
                }
            }
            //去重
            if(isRepitition(oldpar.task)){
                flag = false;
                par = oldpar;
            }
        }
//        }
/*        //排序
        int temp;
        for(int p = 0; p<(c-1); p++) {
            for(int j = 0; j <(c-1-p); j++) {
                if(par.task[j] > par.task[j + 1]) {
                    temp = par.task[j + 1];
                    par.task[j + 1] = par.task[j];
                    par.task[j] = temp;
                }
            }
        }*/

        for(int i = 0; i < c; i++) {
            par.taskfinal[i] = par.task[i];
            if(par.taskfinal[i]>dev){
                System.out.println(421);
            }
        }

    }

    private boolean isRepitition(int[] task) {
        boolean flag = true;
        for (int i = 1;i<task.length-1;i++){
            for (int j = i+1;j<task.length-1;j++){
                if (task[i]==task[j]) {
                    flag = false;
                }
            }
        }
        return flag;
    }
//    }
    /**
     * 计算适应度值
     */
    private double compute_fit(Par par, double[] D, double[][] lt,
                               double[] nt, int c, int dev, int n1, int n2, int nn, int[][] link, int num, double slot) {
        //处理时延
        double[] compu = new double[c];
        //累计时延
        double[] accu = new double[c];
//        //初始化任务数组
//        int[] task = new int[c];
//
//        //extract 提取映射结果
//        task[0] = 0;
//        task[c - 1] = dev - 1;
//        int index = 1;
//        for (int i = 0; i < n1 * n2; i += n2) {
//            for (int j = 0; j < n2; j++) {
//                if (par.x[i + j] == 1) {
//                    task[index] = j;
//                    index++;
//                    break;
//                }
//            }
//        }
//        System.out.println(Arrays.toString(task));
//        //初始化任务数组
//        task = new int[] {0, 7, 4, 4, 7, 7, 11};

        /*
         * 业务处理时延 = 业务累计时延 + 业务计算时延
         * 业务累计时延 = 前项业务处理时延 + 业务传输时延
         *
         * 业务处理时延 = 前项业务处理时延 +（业务传输时延 + 业务计算时延）
         */
        compu[0] = 0;
        accu[0] = 0 + compu[0];

        /*for (int i = 1; i < c; i++) {
            compu[i] = nt[par.taskfinal[i]%num];
        }*/

        //子任务终点
        for (int i = 1; i < c; i++) {
            double max_trans = 0;
            int link_size = link[0].length;
            //子任务起点
            for (int j = 0; j < link_size; j++) {
//                int end = link[1][j];
                int end = par.taskfinal[link[1][j]];
                if ( link[1][j] == i) {
//                    int start = link[0][j];
                    int start = par.taskfinal[link[0][j]];
                    double transv;
                    if (start/num == end/num){
                        transv = accu[link[0][j]] + lt[start][end];
                    }else{
                        transv = (lt[start][end]*D[j])+start/num*slot;  //传输跨时隙的话，时延应该是跨时隙后的时刻
                    }
                    max_trans = Math.max(max_trans, transv);
                }
            }
            compu[i] = D[i]/nt[par.taskfinal[i]%num];
            accu[i] = compu[i] + max_trans;
            //modify
            if(accu[i] > (par.taskfinal[i]/num + 1)*slot) {
                if(par.taskfinal[i] + num < dev){
                    par.taskfinal[i] += num;			//缺少参数 num, slot
                    for(int j = i + 1; j<c ;j++) {
                        if(par.taskfinal[j]/num < par.taskfinal[i]/num) {
                            par.taskfinal[j] += num;
                        }
                    }
                }else {
//                    System.out.println("本方案不可行");
                    return 0;
                }
            }
        }
        double fit = 1 / accu[c - 1];
        return fit;
    }

    /**
     * 粒子引用类型，是粒子群算法搜索的最小单元
     *  输入参数：位置（自变量）、速度、适应度、[个体最优位置、个体最优适应度(解)]
     */
    class Par {
        //粒子群输入参数声明
        int[] x;
        double[] v;
        double fit;
        double best_fit;
        int[] bestx;
        int[] task;
        int[] taskfinal;

        //构造函数
        public Par(int nn,int c) {
            x = new int[nn];
            v = new double[nn];
            bestx = new int[nn];
            fit = 0;
            best_fit = 0;
            task = new int[c];
            taskfinal = new int[c];
        }

    }

}

