package top.ninwoo.cloud.service.cpcTrans;

import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;
import top.ninwoo.api.CnfData;
import top.ninwoo.api.Picture;
import top.ninwoo.cloud.config.BpsoConfig;
import top.ninwoo.cloud.service.IPService;
import top.ninwoo.cloud.service.TransService;
import top.ninwoo.cloud.service.bpso.Bpso;
import top.ninwoo.cloud.service.findService.FindServiceImpl;

import javax.annotation.Resource;
import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.concurrent.ConcurrentHashMap;

import static top.ninwoo.cloud.service.bpso.Bpso.lastHop;

@Slf4j
@Service
public class CpcTrans implements TransService {

    @Resource
    private RestTemplate restTemplate;

    @Autowired
    private BpsoConfig bpsoConfig;

    @Autowired
    private IPService ipService;
    /**
     * 打包picture
     *
     * @return picture列表
     */
    @Override
    public Picture packagePic(BufferedImage bufferedImage){
        Picture picture = new Picture();
        //转化为byte[]数组
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        try {
            //todo 需要增加输入图片格式
            ImageIO.write(bufferedImage, "jpg", out);
        } catch (IOException e) {
            e.printStackTrace();
        }
        //设置图片数据
        picture.setData(out.toByteArray());
        //设置文件名
        picture.setPicId("123");
        return picture;
    }

    /**
     * 打包成数据包
     */
    @Override
    public LinkedList<CnfData> packageData(Picture picture, int[][] link,int[] func) throws ParseException {
        LinkedList<CnfData> cnfDatas = new LinkedList<>();
        FindServiceImpl findService = new FindServiceImpl();
        //判断npe
        if(picture == null){
            System.out.println("出错");
            return null;
        }

        //      输入：
        //子任务依赖关系 边
        //int[][] link = {{0, 1, 1, 2, 3, 4, 5, 6}, {1, 2, 3, 4, 5, 6, 6, 7}};
        //task编号
        //int[] func = {0, 2, 3, 3, 1, 1, 4, 0};
        //任务表
        String[] tasks = {"start/end","binaryProcess", "cutProcess", "greyProcess", "mergeProcess","tailorProcess", "distributeProcess"};
        //卫星名列表
        String[] iridiums = {"IRIDIUM 103",
                "IRIDIUM 102",
                "IRIDIUM 104",
                "IRIDIUM 108",
                "IRIDIUM 113",
                "IRIDIUM 120",
                "IRIDIUM 117",
                "IRIDIUM 128",
                "IRIDIUM 133",
                "IRIDIUM 122",
                "IRIDIUM 119",
                "IRIDIUM 132",
                "IRIDIUM 139",
                "IRIDIUM 135",
                "IRIDIUM 116",
                "IRIDIUM 151",
                "IRIDIUM 137",
                "IRIDIUM 153",
                "IRIDIUM 144",
                "IRIDIUM 157" };


        Bpso doMap = new Bpso();
        doMap.doMapApp(link,func,bpsoConfig);
        //映射结果
        int[] result = doMap.result;
//        System.out.println("optimal mapping scheme"+ Arrays.toString(result));
//        log.info("optimal mapping scheme"+ Arrays.toString(result));

        //路径表
        String[] path = new String[link[0].length];
/*        int sateEndNum = result[result.length - 1];
        int sateStartNum = result[0];*/
        for (int tm = 0;tm<link[0].length;tm++){
            String startHop = "sate"+result[link[0][tm]];
            String endHop = "sate"+result[link[1][tm]];
            if(startHop.equals("sate0")){
                path[tm] = "D1" +"->"+ endHop;
                continue;
            }
            //要修改
            //todo
/*            if(endHop.equals("sate"+sateEndNum)){
                path[tm] = startHop +"->"+ "D2";
                continue;
            }*/
            //左为边起点，右为边终点
            path[tm] = startHop+"->"+endHop;
        }
//        log.info("任务规划结果："+Arrays.toString(path));
//        向展示效果低头
        String[] pathTable = new String[link[0].length];
        for (int tm = 0;tm<link[0].length;tm++) {
//            pathTable[tm] = result
            String[] split = path[tm].split("->");
            StringBuffer stringBuffer = new StringBuffer();
            if (split[0].equals("D1")) {
                stringBuffer.append(iridiums[0]);
            }else stringBuffer.append(iridiums[Integer.parseInt(split[0].split("sate")[1])]);
            stringBuffer.append("->");
            stringBuffer.append(iridiums[Integer.parseInt(split[1].split("sate")[1])]);
            pathTable[tm] = stringBuffer.toString();
        }
        log.info("pathTable: "+Arrays.toString(pathTable));
        System.out.println();

        //任务表
        ConcurrentHashMap<String,String> taskTable = new ConcurrentHashMap();
        for (int tm = 0;tm<result.length;tm++){
            String hop = "sate"+result[tm];
            if(hop.equals("sate0")){
                taskTable.put("D1",tasks[func[tm]]);
                continue;
            }
            //要修改
            //todo
/*            if(hop.equals("sate"+sateEndNum)){
                taskTable.put("D2",tasks[func[tm]]);
                continue;
            }*/
            taskTable.put(hop,tasks[func[tm]]);
        }

//        log.info("子任务功能执行卫星列表:"+taskTable.toString());
        // 向展示效果低头
        ConcurrentHashMap<String,String> taskTableIrid = new ConcurrentHashMap();
        for (int tm = 0;tm<result.length;tm++){
            String hop = iridiums[result[tm]];
            taskTableIrid.put(hop,tasks[func[tm]]);
        }
        log.info("taskTable: "+taskTableIrid.toString());
        System.out.println();
        //T3路由表
        String[][] route = new String[lastHop.length][lastHop[0].length];
        for(int i =0;i<lastHop.length;i++){
            for (int j=0;j<lastHop[0].length;j++){
                route[i][j] = "sate"+(lastHop[i][j]);
                if (route[i][j].equals("sate0")){
                    route[i][j]="sate"+(j);
                }
                if (route[i][j].equals("sate0")){
                    route[i][j]="D1";
                }
                //要修改
                //todo
/*                if (route[i][j].equals("sate"+(sateEndNum-1))){
                    route[i][j]="D2";
                }*/
            }
        }
//        log.info("跨时隙路由表："+Arrays.deepToString(route));
        // 向展示效果低头
        String[][] routeIrid = new String[lastHop.length][lastHop[0].length];
        for(int i =0;i<lastHop.length;i++){
            for (int j=0;j<lastHop[0].length;j++) {
                routeIrid[i][j] = iridiums[lastHop[i][j]];
            }
        }
//        log.info("route: "+Arrays.deepToString(routeIrid));

        boolean flag = false;
//        防止形成自环，比如route[1][2] = sate1 那么发往sate2的数据包将永远在sate1自环
        for(int i =0;i<lastHop.length;i++){
            for (int j=0;j<lastHop[0].length;j++){
                if (route[i][j].equals("sate"+(i)) && i!=j){
                    flag = true;
                }
            }
        }
        if (flag==false){
//            System.out.println("correct");
        }

        CnfData dataPackage = new CnfData();
//        ConcurrentHashMap<String,String> ipool = findService.findIPool();
//        dataPackage.setIpPool(ipool);

        //路径表
        dataPackage.setPathTable(path);
        //任务表
        dataPackage.setTaskTable(taskTable);
        //路由表
        dataPackage.setRoute(route);
        //图片
        dataPackage.setPicture(picture);
        //ip池
//        dataPackage.setIpPool(findService.findIPool());
        String[] routeself = findService.findRoute(dataPackage.getCurrentSataHop(), dataPackage.getRoute());
        log.info("route of current sate: "+ Arrays.toString(routeIrid[0]));
        System.out.println();
        ArrayList<String> destinSataHops = findService.findDestinSataHops(path,"D1");
//        log.info("该子任务边终止卫星："+destinSataHops);

        for(String des : destinSataHops) {
            CnfData transferPackage = dataPackage;
            log.info("next Task exec sate: "+ iridiums[Integer.parseInt(des.split("sate")[1])]);
            System.out.println();
            //目的
            transferPackage.setDestinSataHop(des);
            //当前
            transferPackage.setNextSataHop(findService.findNextHop(des, routeself));
            //下一跳
            transferPackage.setCurrentSataHop(transferPackage.getNextSataHop());
            //ip
            String ip = ipService.getIpByAppName(transferPackage.getNextSataHop());
//            log.info("下一跳ip地址："+ip);
            transferPackage.setNextIPHop(ip);
//            transferPackage.setNextIPHop(findService.findIp(transferPackage.getNextSataHop(),transferPackage.getIpPool()));
            //终止节点
            int SerialsNum = Bpso.num-1;
            transferPackage.setEndSateHop("sate"+SerialsNum);
            //返回一个数据包
            cnfDatas.add(transferPackage);
        }
        return cnfDatas;
    }

    @Override
    public LinkedList<CnfData> packageData(Picture picture, String satename) throws ParseException {
        return null;
    }

    /**
     * 发送数据包
     *
     * @param cnfData
     * @param url
     */
    @Override
    public String sendData(CnfData cnfData, String url) {

        String IP = "172.17.0.2:8083";
//        String IP =url+":8083";
        log.info("nextHopIp: "+url);
        System.out.println();
        ResponseEntity<String> res = restTemplate.postForEntity("http://" + IP + "/transfer", cnfData, String.class);
        if (!res.getStatusCode().is2xxSuccessful()) {
            throw new RuntimeException("send Error!");
        }
        return "success";
    }

    /**
     * 发送得到cpu计算能力的指令
     *
     * @param url
     */
    @Override
    public Double sendCpuCtr(String url) {
        String IP = url + ":8083";
        return restTemplate.getForObject("http://" + IP + "/getCpuComputing", Double.class);
    }

    /**
     * 得到稳态图
     */
    @Override
    public String[][] getIt() throws ParseException {
        return new String[0][];
    }


    private boolean repetition(int[] result) {
        HashSet<Integer> set = new HashSet<>();
        for (int a : result){
            set.add(a);
        }
        if (set.size()==result.length) return false;
        else return true;
    }

    @Override
    public String reset(String url) {
        String IP = url + ":8083";
        ResponseEntity<String> res = restTemplate.getForEntity("http://" + IP + "/reset", String.class);
        if (!res.getStatusCode().is2xxSuccessful()) {
            throw new RuntimeException("send Error!");
        }
        return "sucess";
    }
}
