package top.ninwoo.edge.service.impl;


import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;
import top.ninwoo.api.CnfData;
import top.ninwoo.api.Picture;
import top.ninwoo.api.TacticsCtl;
import top.ninwoo.edge.service.FindService;
import top.ninwoo.edge.service.Tactics;
import top.ninwoo.edge.service.impl.transferService.TransServiceImpl;

import javax.annotation.Resource;
import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.concurrent.ConcurrentHashMap;

@Service
public class TacticsImpl implements Tactics {

    private int level;

    private static int slicenum = 7;

    private CnfData oriData = new CnfData();

    private String currentSate;

    //阻止更多的信令回来
    private int tag;

    //返回的卫星和计算能力
    private ConcurrentHashMap<String, Double> computeTable = new ConcurrentHashMap<>();

    @Resource
    private RestTemplate restTemplate;

    @Autowired
    private FindService findService;

    @Autowired
    TransServiceImpl transService;
    /**
     * 打包成信令
     */
    @Override
    public TacticsCtl packageTactics(CnfData cnfData) {
        //初始化一些参数
        init(cnfData);
        //打包
        TacticsCtl tacticsCtl = new TacticsCtl();
        tacticsCtl.setSate(cnfData.getCurrentSataHop());
        tacticsCtl.setLt(cnfData.getLt());
        tacticsCtl.setIpPool(cnfData.getIpPool());
        tacticsCtl.setRoute(cnfData.getRoute());

        setLevel(tacticsCtl);

        return tacticsCtl;
    }
    /**
     * 请求信令
     */
    @Override
    public TacticsCtl askTactics(TacticsCtl tacticsCtl) {
        if(setLevel(tacticsCtl)){
            return repackage(tacticsCtl);
        }
        return null;
    }

    /**
     * repackage
     */
    @Override
    public TacticsCtl repackage(TacticsCtl tacticsCtl) {
        ConcurrentHashMap<String, Double> sateMap = new ConcurrentHashMap<>();

        Double res = sendCpuCtr("127.0.0.1");
        sateMap.put(tacticsCtl.getCurrentHop(), res);
        tacticsCtl.setSateMap(sateMap);
        return tacticsCtl;
    }

    /**
     * 处理信令
     */
    @Override
    public String handle() throws IOException {
        ConcurrentHashMap<String, Picture> result = imageSplit(oriData.getPicture(),computeTable);
        for (String key : result.keySet()) {
            CnfData cnfData = oriData;

            System.out.println(key+" : "+result.get(key).getSliceId());

            System.out.println("currenthop: "+ currentSate);

            String[] path = new String[2];
            path[0] = currentSate +"->"+key;

            System.out.println(path[0]);

            path[1] = key + "->" + currentSate;

            System.out.println(path[1]);

            //路径表
            cnfData.setPathTable(path);
            ConcurrentHashMap<String, String> tasktable = new ConcurrentHashMap<>();
            tasktable.put(key,"greyProcess");
            tasktable.put(currentSate,"start/end");
            //任务表
            cnfData.setTaskTable(tasktable);
            //设置图片
            cnfData.setPicture(result.get(key));
            //设置目的节点
            cnfData.setDestinSataHop(key);

            System.out.println("destinsatahop "+key);

            String[] route = findService.findRoute(currentSate, cnfData.getRoute());
            //设置下一跳地址
            cnfData.setNextSataHop(findService.findNextHop(cnfData.getDestinSataHop(), route));

            //设置本机地址
            cnfData.setCurrentSataHop(cnfData.getNextSataHop());
            //设置下一跳ip地址
            String nextIP = findService.findIp(cnfData.getNextSataHop(), cnfData.getIpPool());
            cnfData.setNextIPHop(nextIP);
            System.out.println("send picRes to "+nextIP);
            //发包
            transService.sendData(cnfData, cnfData.getNextIPHop());
        }
        return "sucess";
    }

    @Override
    public String setTactics(TacticsCtl tacticsCtl) {
        if(tag==1) {
            for (String key : tacticsCtl.getSateMap().keySet()) {
                System.out.println(key+":"+tacticsCtl.getSateMap().get(key));
                computeTable.put(key, tacticsCtl.getSateMap().get(key));
            }
            if (computeTable.size() == slicenum) {
                tag = 0;
                System.out.println("complete receive");
                return "complete";
            }
        }
        return "wait";
    }

    //得到本机的计算能力
    public Double sendCpuCtr(String url) {
        //System.out.println("kaishi fasong ");
        String IP = url + ":8083";
        Double res = restTemplate.getForObject("http://" + IP + "/getCpuComputing", Double.class);
        //System.out.println(IP+":"+res);
        return res;
    }

    private boolean setLevel(TacticsCtl tacticsCtl){
        //level == 0 等于没有被扩散
        if(level == 0){
            System.out.println("no distributed");
            level = tacticsCtl.getLevel();
            System.out.println("kuosandengji:"+level);
            return true;
        }
        System.out.println("start distributed");
        return false;
    }

    private void init(CnfData cnfData){
        level = 0;
        tag = 1;
        oriData = cnfData;
        currentSate = cnfData.getCurrentSataHop();
    }

    private ConcurrentHashMap<String, Picture> imageSplit(Picture picture, ConcurrentHashMap<String, Double> computeTable) throws IOException {
        ConcurrentHashMap<String, Picture> imgstable = new ConcurrentHashMap<>();
        //归一化处理
        ConcurrentHashMap<String, Double> compute = normalize(computeTable);
        //BufferedImage images = readImage(name);
        ByteArrayInputStream bis = new ByteArrayInputStream(picture.getData());

        BufferedImage images = ImageIO.read(bis);
        int rows = 1;

        // 计算每个小图的宽度和高度
        int chunkHeight = images.getHeight() / rows;

        int count = 0;
        int tmp = 0;
        //BufferedImage imgs[] = new BufferedImage[chunks];
        BufferedImage imgs[] = new BufferedImage[slicenum];
        for (String key : compute.keySet()) {
            int chunkWidth = (int) (images.getWidth()*compute.get(key));
            //设置小图的大小和类型
            imgs[count] = new BufferedImage(chunkWidth, chunkHeight, images.getType());
            //写入图像内容
            Graphics2D gr = imgs[count++].createGraphics();
            gr.drawImage(images, 0, 0,
                    chunkWidth, chunkHeight,
                    tmp, 0,
                    tmp + chunkWidth, chunkHeight,
                    null);
            gr.dispose();
            tmp = tmp + chunkWidth;
        }
        //将imgs[]转为相应的byte数组
        int i = 0 ;
        for (String key : compute.keySet()) {
            ByteArrayOutputStream bos = new ByteArrayOutputStream();
            try {
                ImageIO.write(imgs[i],"jpg",bos);
            } catch (IOException e) {
                e.printStackTrace();
            }
            try {
                bos.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
            Picture picture1 = new Picture();
            picture1.setData(bos.toByteArray());
            picture1.setSliceNum(slicenum);
            picture1.setPicId(picture.getPicId());
            picture1.setSliceId(i+"");
            imgstable.put(key,picture1);
            i++;
        }
        return imgstable;
    }

    //归一化的结果
    private ConcurrentHashMap<String, Double> normalize(ConcurrentHashMap<String, Double> computeTable){
        ConcurrentHashMap<String, Double> normalizeTable = new ConcurrentHashMap<>();
        double sum = 0.0;
        for (String key : computeTable.keySet()) {
            sum = sum +computeTable.get(key);
        }
        for (String key : computeTable.keySet()) {
            double after = computeTable.get(key)/sum;
            normalizeTable.put(key,after);
            //System.out.println(key+":"+normalizeTable.get(key));
        }

        return normalizeTable;
    }
}
