package top.ninwoo.edge.service.impl.transferService;


import org.springframework.stereotype.Service;
import top.ninwoo.edge.service.FindService;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

@Service
public class FindServiceImpl implements FindService {
    /**
     * 根据路径表，找到本机需要处理路径条数
     * c->a ==>  sata1->sata3
     *
     * @param pathTable
     * @param currenthop
     */
    @Override
    public ArrayList<String> findDestinSataHops(String[] pathTable, String currenthop) {
        ArrayList<String> destinSataHops = new ArrayList<String>();
        int j = 0;

        if(pathTable.length == 0){
            System.out.println("路径表为空");
            destinSataHops.add("nopara");
            return destinSataHops;
        }
        for (int i = 0; i < pathTable.length; i++) {
            //判断本机是否为路径起点
            if(currenthop.equals(pathTable[i].split("->")[0])){
                destinSataHops.add(pathTable[i].split("->")[1]);
            }
        }
        return destinSataHops;
    }

    /**
     * 根据本机卫星名，全局路由表,路由表得到本机路由表
     *
     * @param currentSataHop
     * @param routeTable
     */
    @Override
    public String[] findRoute(String currentSataHop, String[][] routeTable) {
        //判断路由表是否存在
        if(routeTable.length == 0){
            System.out.println("路由表为空");
            return null;
        }
        //找到本机路由表
        String[] route = this.findCurrentRoute(currentSataHop,routeTable);
        return route;
    }

    /**
     * 根据目的卫星名，本机路由表找到下一跳卫星名
     *
     * @param destinSataHop
     * @param route
     */
    @Override
    public String findNextHop(String destinSataHop, String[] route) {
        int index = Integer.parseInt(destinSataHop.split("sate")[1]);
        if(route[index].equals("direct")){
            return destinSataHop;
        }
        if(route[index].equals("self")){
            return destinSataHop;
        }
        return route[index];
    }

    /**
     * 找到本机的任务
     *
     * @param currenSataHop
     * @param taskMap
     */
    @Override
    public String findTask(String currenSataHop, ConcurrentHashMap<String, String> taskMap) {
        if(taskMap.isEmpty()){
            System.out.println("无任务表");
            return null;
        }
        return taskMap.get(currenSataHop);
    }

    /**
     * 找到下一跳的ip地址
     *
     * @param sataName
     * @return
     */
    // todo 需要设置文件地址
    @Override
    public String findIp(String sataName, ConcurrentHashMap<String, String> ipool) {
        String sataIP = ipool.get(sataName);
        if(sataIP.equals(null)){
            return "no ip";
        }
        return sataIP;
    }

    /**
     * 找到直联的节点
     */
    @Override
    public Set<String> findDrictHop(String currenthop, String[][] routeTable) {
        //System.out.println(currenthop);
        int num = routeTable.length / 3;
        String[] route = findRoute(currenthop, routeTable);
        Set<String> drictHop = new HashSet<>();
        for (int i = 0; i < route.length; i++) {
            if (route[i] != null && route[i].equals("direct") && !currenthop.equals("sate" + (i % num))) {
                //System.out.println(i);
                drictHop.add("sate" + (i % num));
            }
        }
        return drictHop;
    }

    @Override
    public int findBackTime(String currenthop, String dishop, String[][] route) {
        //卫星数目
        int num = route.length / 3;
        //时隙数目
        int time = 0;

        int index = Integer.parseInt(dishop.split("sate")[1]);
        String[] rouTable = findCurrentRoute(currenthop, route);
        //每一个时隙都去判断
        for (int i = 0; i<3; i++) {
            if ( ((num*i) +index) < rouTable.length || rouTable[(num*i) +index] != null){
                time = i;
            }
            else break;
        }
        return time;
    }

    //找到本卫星的路由
    private String[] findCurrentRoute(String sataname, String[][] routeTable){
//        这里减1是为了将卫星真实编号与索引值对应
        int index = Integer.parseInt(sataname.split("sate")[1]);
//        System.out.println(index);
        return routeTable[index];
    }


}
