package top.ninwoo.edge.service.impl.transferService;


import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;
import top.ninwoo.api.CnfData;
import top.ninwoo.api.Picture;
import top.ninwoo.api.TacticsCtl;
import top.ninwoo.edge.service.FindService;
import top.ninwoo.edge.service.IPService;
import top.ninwoo.edge.service.TransferService;
import top.ninwoo.edge.service.impl.PicServiceImpl;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.LinkedList;

@Service
@Slf4j
public class TransServiceImpl implements TransferService {

    @Resource
    private RestTemplate restTemplate;

    @Autowired
    private FindService findService;

    @Autowired
    private PicServiceImpl picService;

    @Autowired
    private IPService ipService;


    @Override
    public String sendData(CnfData cnfData, String url){
        String IP = url + ":8083";
        ResponseEntity<String> res = restTemplate.postForEntity("http://" + IP + "/transfer", cnfData, String.class);
        if (!res.getStatusCode().is2xxSuccessful()) {
            throw new RuntimeException("send Error!");
        }
//        System.out.println("send picRes sucess");
        return "success";
    }

    /**
     * 发送指令
     */
    @Override
    public String sendCtl(TacticsCtl tacticsCtl, String url) {
        String IP = url + ":8083";
        ResponseEntity<String> res = restTemplate.postForEntity("http://" + IP + "/tactics", tacticsCtl, String.class);
        if (!res.getStatusCode().is2xxSuccessful()) {
            throw new RuntimeException("send Error!");
        }
        System.out.println("send ctl sucess");
        return "sucess";
    }

    @Override
    public String backCtl(TacticsCtl tacticsCtl, String url) {
        String IP = url + ":8083";
        ResponseEntity<String> res = restTemplate.postForEntity("http://" + IP + "/backtactics", tacticsCtl, String.class);
        if (!res.getStatusCode().is2xxSuccessful()) {
            throw new RuntimeException("send Error!");
        }
        System.out.println("back ctl sucess");
        return "sucess";
    }

    @Override
    public String parseData(CnfData cnfData) {

        //npe检查
        if(cnfData.equals(null)){
            return "error";
        }

        //是否为终节点
        if(cnfData.getCurrentSataHop().equals(cnfData.getEndSateHop())) {
            return "end";
        }

        //是否为该边节点
        if (cnfData.getCurrentSataHop().equals(cnfData.getDestinSataHop())){
//            通过任务分配表和当前卫星名寻找当前任务
            String task = findService.findTask(cnfData.getCurrentSataHop(),cnfData.getTaskTable());
//            任务不在列表中
            if (task.equals("null")){
                System.out.println("find error");
                return "error";
            }
            if (task.equals("start/end")){
                return "end";
            }
//            啥意思
            if(task.equals("distributeProcess")){
                return "distributeProcess";
            }
//            传输当前任务名及待处理图片，返回节点执行状态
            String status = picService.setPicOrigin(task, cnfData.getPicture());
            switch (status) {
                case "success":
                    //正义执行
                    return "pathComputing";
                case "wait":
                    //等待执行
                    return "wait";
                case "error":
                    //报错
                    return "error";
            }
        }
        //转发节点
        else {
            return "pass";
        }
        return "i dont know what error";
    }

    /**
     * 装包
     */
    @Override
    public LinkedList<CnfData> packageData(CnfData cnfData) {

        LinkedList<CnfData> newData = new LinkedList<>();
        //找到路由
        String[] route = findService.findRoute(cnfData.getCurrentSataHop(), cnfData.getRoute());
//        log.info("current sate route: "+ Arrays.toString(route));
        CnfData newDataPackage = cnfData;
        //设置下一跳地址
        newDataPackage.setNextSataHop(findService.findNextHop(newDataPackage.getDestinSataHop(), route));
        //设置本机地址
        newDataPackage.setCurrentSataHop(newDataPackage.getNextSataHop());
        //设置下一跳ip地址
        String nextIP = ipService.getIpByAppName(newDataPackage.getNextSataHop());
//        String nextIP = findService.findIp(newDataPackage.getNextSataHop(),newDataPackage.getIpPool());
        newDataPackage.setNextIPHop(nextIP);
        newData.add(newDataPackage);
        return newData;
    }

    @Override
    public LinkedList<CnfData> packageData(CnfData cnfData, LinkedList<Picture> pictures) {
        LinkedList<CnfData> newData = new LinkedList<>();
        //找到路由
        String[] route = findService.findRoute(cnfData.getCurrentSataHop(), cnfData.getRoute());
        //找到每段的终节点
        ArrayList<String> destinSataHops = findService.findDestinSataHops(cnfData.getPathTable(), cnfData.getCurrentSataHop());
        for (String destinSataHop : destinSataHops) {
            CnfData newDataPackage = new CnfData();
            //重新设置
            newDataPackage.setEndSateHop(cnfData.getEndSateHop());
            newDataPackage.setTaskTable(cnfData.getTaskTable());
            newDataPackage.setRoute(cnfData.getRoute());
            newDataPackage.setPathTable(cnfData.getPathTable());
//            newDataPackage.setIpPool(cnfData.getIpPool() );
            //设置目的地址
            newDataPackage.setDestinSataHop(destinSataHop);
            //设置下一跳地址
            newDataPackage.setNextSataHop(findService.findNextHop(destinSataHop, route));
            //设置本机地址
            newDataPackage.setCurrentSataHop(newDataPackage.getNextSataHop());
            //设置下一跳ip地址
            String nextIP =ipService.getIpByAppName(newDataPackage.getNextSataHop());
//            String nextIP = findService.findIp(newDataPackage.getNextSataHop(),newDataPackage.getIpPool());
            newDataPackage.setNextIPHop(nextIP);
            //放入图片
            newDataPackage.setPicture(pictures.removeFirst());
            newData.add(newDataPackage);
        }
        return newData;
    }

    @Override
    public void backEarth(String message){
        String IP = "192.168.253.130" + ":8082";
        String res = restTemplate.postForObject("http://" + IP + "/getDate?message=", message, String.class);
//        System.out.println(res);
    }
}
