package top.ninwoo;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;
import top.ninwoo.bishe.starter.entity.ContainerMonitor;
import top.ninwoo.bishe.starter.service.ClusterService;
import top.ninwoo.bishe.starter.service.NetworkService;
import top.ninwoo.common.entity.*;
import top.ninwoo.weixingsim.WxMain;
import top.ninwoo.weixingsim.entity.SimData;
import top.ninwoo.weixingsim.entity.WeiXingData;
import top.ninwoo.weixingsim.service.impl.ToponetImpl;
import top.ninwoo.weixingsim.service.impl.WeixingImpl;

import javax.annotation.Resource;
import java.io.File;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;

@RunWith(SpringRunner.class)
@SpringBootTest(classes = WxMain.class)
public class WxTests {

    @Autowired
    private ClusterService clusterService;

    @Resource
    private NetworkService networkService;

    @Test
    public void testSendSeparateConfig() {
        ArrayList<SeparatedClusterConfig> clusterConfigs = new ArrayList<>();
        SeparatedClusterConfig separatedClusterConfig = new SeparatedClusterConfig();
        // TODO: 这个ID应该是从借口获取的
        separatedClusterConfig.setEdgeNodeId("192.168.190.135:8081");
        ClusterConfig clusterConfig = new ClusterConfig();
        clusterConfig.setId(11111l);
        clusterConfig.setOwner("joliu");
        ContainerDescription containerDescription = new ContainerDescription();
        containerDescription.setMode("normal");
        containerDescription.setReplicas(2);
        DockerContainer container = new DockerContainer();
        container.setName("Run");
        container.setCommand("sh");
        container.setImage("joliu/networktest");
        containerDescription.setDockerContainer(container);

        List<ContainerDescription> cds = new ArrayList<>();
        cds.add(containerDescription);

        ContainerDescription containerDescription1 = new ContainerDescription();
        containerDescription1.setMode("normal");
        containerDescription1.setReplicas(3);
        DockerContainer container1 = new DockerContainer();
        container1.setName("APP");
        container1.setCommand("sh");
        container1.setImage("joliu/networktest");
        containerDescription1.setDockerContainer(container1);

        cds.add(containerDescription1);

        clusterConfig.setDockers(cds);
        NetworkTopology topo = new NetworkTopology();
        topo.setAppNames(new String[]{"Run", "APP", "br:ovs1"});
        // 这个参数好像没啥用
        topo.setTopologyId(11);
        topo.setTopology(new int[][]{{0,0,0},{0,0,0},{1,1,0}});
        clusterConfig.setTopology(topo);
        separatedClusterConfig.setClusterConfig(clusterConfig);
        clusterConfigs.add(separatedClusterConfig);

        clusterService.sendClusterConfigToEdgeNode(clusterConfigs);
    }

    @Test
    public void removeClusterFromEdgeNodeTest() {
        clusterService.removeClusterFromEdgeNode(11111l);
    }

    @Test
    public void getLogicalNetworkTopologyTest() {
        NetworkTopology logicalNetworkTopology = networkService.getLogicalNetworkTopology(11111L);
        System.out.println(logicalNetworkTopology);
    }

    @Test
    public void getIpListByAppNameTest() {
        List<String> ipList = networkService.getIpListByAppName(11111L, "Run");
        System.out.println(ipList);
    }

    @Test
    public void enableNetworkMonitorTest() throws InterruptedException {
        String res = networkService.enableNetworkMonitor(11111L, "Run");
        System.out.println(res);
    }

    @Test
    public void cancelNetworkMonitorTest() {
        String run = networkService.cancelNetworkMonitor(11111L, "Run");
        System.out.println(run);
    }

    @Test
    public void getNetworkMonitorTest() throws InterruptedException {
        List<ContainerMonitor> run = networkService.getContainerMonitors(11111L, "Run");
        int i = 0;
        while(i < 10) {
            run.forEach(c -> {
                System.out.println(c.getContainerInfo());
            });
            Thread.sleep(500);
            i++;
        }
    }

    //改变集群拓扑配置
    private boolean adjustCluster(List<SeparatedClusterConfig> configs) {
        boolean b = clusterService.adjustClusterToEdgeNode(configs);
        return b;
    }

    //测试逻辑网路
    @Test
    public void test1() {
        ArrayList<SeparatedClusterConfig> clusterConfigs = new ArrayList<>();
        SeparatedClusterConfig separatedClusterConfig = new SeparatedClusterConfig();
        //需修改为下发的IP地址
        separatedClusterConfig.setEdgeNodeId("192.168.190.135:8081");

        ClusterConfig clusterConfig = new ClusterConfig();
        clusterConfig.setId(11111l);
        clusterConfig.setOwner("joliu");
        ContainerDescription containerDescription = new ContainerDescription();
        containerDescription.setMode("normal");
        containerDescription.setReplicas(1);
        DockerContainer container = new DockerContainer();
        container.setName("D1");
        container.setCommand("sh");
        container.setImage("test2");
        containerDescription.setDockerContainer(container);

        List<ContainerDescription> cds = new ArrayList<>();
        cds.add(containerDescription);

        ContainerDescription dis = new ContainerDescription();
        dis.setMode("normal");
        dis.setReplicas(1);
        DockerContainer disContainer = new DockerContainer();
        disContainer.setName("D2");
        disContainer.setCommand("sh");
        disContainer.setImage("test2");
        dis.setDockerContainer(disContainer);
        cds.add(dis);

        ContainerDescription dis2 = new ContainerDescription();
        dis2.setMode("normal");
        dis2.setReplicas(1);
        DockerContainer disContainer2 = new DockerContainer();
        disContainer2.setName("D3");
        disContainer2.setCommand("sh");
        disContainer2.setImage("test2");
        dis2.setDockerContainer(disContainer2);
        cds.add(dis2);

        String[] appnames = new String[]{"D1", "D2", "D3", "br:ovs"};
        int[][] topology = new int[][]{
                {0, 0, 0 ,0},
                {1, 0, 0 ,0},
                {0, 1, 0 ,0},
                {1, 1, 1 ,0}};

        clusterConfig.setDockers(cds);
        NetworkTopology topo = new NetworkTopology();
        topo.setAppNames(appnames);
        // 这个参数好像没啥用
        topo.setTopologyId(11);
        topo.setTopology(topology);
        clusterConfig.setTopology(topo);
        separatedClusterConfig.setClusterConfig(clusterConfig);
        clusterConfigs.add(separatedClusterConfig);

        clusterService.sendClusterConfigToEdgeNode(clusterConfigs);

        //下发逻辑拓扑
        clusterService.sendLogicTopoToEdgeNode(clusterConfigs);

        /*
        //改变逻辑拓扑
        int[][] newtopology = new int[][]{
                {0, 0, 0, 0 },
                {1, 0, 0, 0 },
                {1, 1, 0, 0 },
                {1, 1, 1, 0 }};
        topo.setTopology(newtopology);

        //下发修改后的逻辑拓扑
        clusterService.adjustLogicTopoToEdgeNode(clusterConfigs);

        */

    }

    // 测试初始化设置
    @Test
    public void test2() throws IllegalAccessException, InterruptedException {
        //从文本中获取卫星的数据
        WeixingImpl wx = new WeixingImpl();
        ToponetImpl tp = new ToponetImpl();
        SimData sd = new SimData();

        File file = new File("C:\\WorkSpace\\test\\topusim_1.txt");
        List<WeiXingData> wxData = wx.iniTopo(file,sd.getZhouqi());
        int[][] toponet = tp.getTopology(wxData,sd);


        ArrayList<SeparatedClusterConfig> clusterConfigs = new ArrayList<>();
        SeparatedClusterConfig separatedClusterConfig = new SeparatedClusterConfig();
        List<ContainerDescription> cds = new ArrayList<>();
        // TODO: 这个ID应该是从借口获取的
        separatedClusterConfig.setEdgeNodeId("192.168.190.135:8081");
        ClusterConfig clusterConfig = new ClusterConfig();
        clusterConfig.setId(11111l);
        clusterConfig.setOwner("joliu");

        //创建容器
        for (int k = 0; k < wxData.size(); k++) {
            ContainerDescription containerDescription = new ContainerDescription();
            containerDescription.setMode("normal");
            containerDescription.setReplicas(1);

            DockerContainer container = new DockerContainer();
            container.setName(wxData.get(k).getName());
            container.setCommand("sh");
            container.setImage("joliu/networktest");

            containerDescription.setDockerContainer(container);
            cds.add(containerDescription);
        }

        clusterConfig.setDockers(cds);
        NetworkTopology topo = new NetworkTopology();
        topo.setAppNames(tp.getAppNames(wxData));
        // 这个参数好像没啥用
        topo.setTopologyId(11);
        topo.setTopology(toponet);
        clusterConfig.setTopology(topo);
        separatedClusterConfig.setClusterConfig(clusterConfig);
        clusterConfigs.add(separatedClusterConfig);

        clusterService.sendClusterConfigToEdgeNode(clusterConfigs);

        //下发逻辑拓扑
        clusterService.sendLogicTopoToEdgeNode(clusterConfigs);
    }

    //测试连续传输
    //测试生成所有的逻辑topo
    @Test
    public void test3() throws IllegalAccessException, InterruptedException {
        ArrayList<SeparatedClusterConfig> clusterConfigs = new ArrayList<>();
        SeparatedClusterConfig separatedClusterConfig = new SeparatedClusterConfig();
        List<ContainerDescription> cds = new ArrayList<>();
        // TODO: 这个ID应该是从借口获取的
        separatedClusterConfig.setEdgeNodeId("192.168.31.198:18088");
        ClusterConfig clusterConfig = new ClusterConfig();
        clusterConfig.setId(11111l);
        clusterConfig.setOwner("joliu");

        //从文本中获取卫星的数据
        WeixingImpl wx = new WeixingImpl();
        ToponetImpl tp = new ToponetImpl();
        SimData sd = new SimData();
        //tp.delHistory();
        File file = new File("D:\\LabratoryJavaPro\\codingDownload\\latestCoding\\topusim_1.txt");
//        D:\LabratoryJavaPro\codingDownload\latestCoding\
        List<WeiXingData> wxData = wx.iniTopo(file,sd.getZhouqi());
        int[][] toponet = tp.getTopology(wxData,sd);


        //创建容器
        ContainerDescription containerDescription0 = new ContainerDescription();
        containerDescription0.setMode("normal");
        containerDescription0.setReplicas(1);

        DockerContainer container0 = new DockerContainer();
        container0.setName(wxData.get(0).getName());
        container0.setCommand("sh");
        container0.setImage("joliu/networktest:schedule");

        containerDescription0.setDockerContainer(container0);
        cds.add(containerDescription0);
        for (int k = 1; k < wxData.size(); k++) {
            ContainerDescription containerDescription = new ContainerDescription();
            containerDescription.setMode("normal");
            containerDescription.setReplicas(1);

            DockerContainer container = new DockerContainer();
            container.setName(wxData.get(k).getName());
            container.setCommand("sh");
            container.setImage("joliu/networktest:server");

            containerDescription.setDockerContainer(container);
            cds.add(containerDescription);
        }

        clusterConfig.setDockers(cds);
        NetworkTopology topo = new NetworkTopology();
        topo.setAppNames(tp.getAppNames(wxData));
        // 这个参数好像没啥用
        topo.setTopologyId(11);
        topo.setTopology(toponet);
        clusterConfig.setTopology(topo);
        separatedClusterConfig.setClusterConfig(clusterConfig);
        clusterConfigs.add(separatedClusterConfig);

        clusterService.sendClusterConfigToEdgeNode(clusterConfigs);

       /* //下发逻辑拓扑
        clusterService.sendLogicTopoToEdgeNode(clusterConfigs);

        out:for (int i = 1; i<600; i++) {
            Thread.sleep(2000);
            int time = i * sd.getJiange();
            List<WeiXingData> changetp = wx.changeTopo(wxData,time);

            topo.setTopology(tp.getTopology(changetp,sd));

            //下发修改后逻辑拓扑
            clusterService.adjustLogicTopoToEdgeNode(clusterConfigs);
            Boolean tag = false;

            for(int k=1;k<toponet[0].length;k++){
                for (int j=0;j<k;j++){
                    if(toponet[k][j] == tp.getTopology(changetp, sd)[k][j]){
                        tag = true;
                    }
                }
            }
            if(tag){
                break out;
            }
        }

        System.out.println("所有topo完成");*/
    }

    //测试历史功能，需要提前定义好输出文本地址
    @Test
    public void test4() throws IllegalAccessException, InterruptedException {
        ArrayList<SeparatedClusterConfig> clusterConfigs = new ArrayList<>();
        SeparatedClusterConfig separatedClusterConfig = new SeparatedClusterConfig();
        List<ContainerDescription> cds = new ArrayList<>();
        // TODO: 这个ID应该是从借口获取的
        separatedClusterConfig.setEdgeNodeId("192.168.190.135:8081");
        ClusterConfig clusterConfig = new ClusterConfig();
        clusterConfig.setId(11111l);
        clusterConfig.setOwner("joliu");

        //从文本中获取卫星的数据
        WeixingImpl wx = new WeixingImpl();
        ToponetImpl tp = new ToponetImpl();
        SimData sd = new SimData();
        tp.delHistory();
        File file = new File("C:\\WorkSpace\\test\\topusim_1.txt");
        List<WeiXingData> wxData = wx.iniTopo(file,sd.getZhouqi());
        int[][] toponet = tp.getTopology(wxData,sd);


        //创建容器
        for (int k = 0; k < wxData.size(); k++) {
            ContainerDescription containerDescription = new ContainerDescription();
            containerDescription.setMode("normal");
            containerDescription.setReplicas(1);

            DockerContainer container = new DockerContainer();
            container.setName(wxData.get(k).getName());
            container.setCommand("sh");
            container.setImage("joliu/networktest");

            containerDescription.setDockerContainer(container);
            cds.add(containerDescription);
        }

        clusterConfig.setDockers(cds);
        NetworkTopology topo = new NetworkTopology();
        topo.setAppNames(tp.getAppNames(wxData));
        // 这个参数好像没啥用
        topo.setTopologyId(11);
        topo.setTopology(toponet);
        clusterConfig.setTopology(topo);
        separatedClusterConfig.setClusterConfig(clusterConfig);
        clusterConfigs.add(separatedClusterConfig);

        clusterService.sendClusterConfigToEdgeNode(clusterConfigs);

        //下发逻辑拓扑
        clusterService.sendLogicTopoToEdgeNode(clusterConfigs);

        for (int i = 1; ; i++) {
            Thread.sleep(2000);
            int time = i * sd.getJiange();
            List<WeiXingData> changetp = wx.changeTopo(wxData,time);

            topo.setTopology(tp.getTopology(changetp,sd));

            tp.history(topo.getAppNames(),topo.getTopology());
            //下发修改后逻辑拓扑
            clusterService.adjustLogicTopoToEdgeNode(clusterConfigs);

        }
    }

    //测试容器运行程序
    @Test
    public void test5() {
        ArrayList<SeparatedClusterConfig> clusterConfigs = new ArrayList<>();
        SeparatedClusterConfig separatedClusterConfig = new SeparatedClusterConfig();
        //需修改为下发的IP地址
        separatedClusterConfig.setEdgeNodeId("192.168.190.135:18088");

        ClusterConfig clusterConfig = new ClusterConfig();
        clusterConfig.setId(11111l);
        clusterConfig.setOwner("joliu");
        ContainerDescription containerDescription = new ContainerDescription();
        containerDescription.setMode("normal");
        containerDescription.setReplicas(1);
        DockerContainer container = new DockerContainer();
        container.setName("D1");
        container.setCommand("-e PARAMS=\"--name=" +container.getName()+"\"");
        container.setImage("joliu/networktest");
        containerDescription.setDockerContainer(container);

        List<ContainerDescription> cds = new ArrayList<>();
        cds.add(containerDescription);

        ContainerDescription dis = new ContainerDescription();
        dis.setMode("normal");
        dis.setReplicas(1);
        DockerContainer disContainer = new DockerContainer();
        disContainer.setName("D2");
        disContainer.setCommand("-e PARAMS=\"--name=" +disContainer.getName()+"\"");
        disContainer.setImage("joliu/networktest");
        dis.setDockerContainer(disContainer);
        cds.add(dis);

        ContainerDescription dis2 = new ContainerDescription();
        dis2.setMode("normal");
        dis2.setReplicas(1);
        DockerContainer disContainer2 = new DockerContainer();
        disContainer2.setName("D3");
        disContainer2.setCommand("-e PARAMS=\"--name=" +disContainer2.getName()+"\"");
        disContainer2.setImage("joliu/networktest");
        dis2.setDockerContainer(disContainer2);
        cds.add(dis2);

        String[] appnames = new String[]{"D1", "D2", "D3", "br:ovs"};
        int[][] topology = new int[][]{
                {0, 0, 0 ,0},
                {1, 0, 0 ,0},
                {0, 1, 0 ,0},
                {1, 1, 1 ,0}};

        clusterConfig.setDockers(cds);
        NetworkTopology topo = new NetworkTopology();
        topo.setAppNames(appnames);
        // 这个参数好像没啥用
        topo.setTopologyId(11);
        topo.setTopology(topology);
        clusterConfig.setTopology(topo);
        separatedClusterConfig.setClusterConfig(clusterConfig);
        clusterConfigs.add(separatedClusterConfig);

        clusterService.sendClusterConfigToEdgeNode(clusterConfigs);

        //下发逻辑拓扑
        clusterService.sendLogicTopoToEdgeNode(clusterConfigs);

        /*
        //改变逻辑拓扑
        int[][] newtopology = new int[][]{
                {0, 0, 0, 0 },
                {1, 0, 0, 0 },
                {1, 1, 0, 0 },
                {1, 1, 1, 0 }};
        topo.setTopology(newtopology);

        //下发修改后的逻辑拓扑
        clusterService.adjustLogicTopoToEdgeNode(clusterConfigs);

        */

    }

    @Test
    public void test6() {
        ArrayList<SeparatedClusterConfig> clusterConfigs = new ArrayList<>();
        SeparatedClusterConfig separatedClusterConfig = new SeparatedClusterConfig();
        //需修改为下发的IP地址
        separatedClusterConfig.setEdgeNodeId("192.168.190.135:18088");

        ClusterConfig clusterConfig = new ClusterConfig();
        clusterConfig.setId(11111l);
        clusterConfig.setOwner("joliu");
        ContainerDescription containerDescription = new ContainerDescription();
        containerDescription.setMode("normal");
        containerDescription.setReplicas(1);
        DockerContainer container = new DockerContainer();
        container.setName("D1");
        container.setCommand("sh");
        container.setImage("test2");
        containerDescription.setDockerContainer(container);

        List<ContainerDescription> cds = new ArrayList<>();
        cds.add(containerDescription);

        ContainerDescription dis = new ContainerDescription();
        dis.setMode("normal");
        dis.setReplicas(1);
        DockerContainer disContainer = new DockerContainer();
        disContainer.setName("D2");
        disContainer.setCommand("sh");
        disContainer.setImage("test2");
        dis.setDockerContainer(disContainer);
        cds.add(dis);

        ContainerDescription dis2 = new ContainerDescription();
        dis2.setMode("normal");
        dis2.setReplicas(1);
        DockerContainer disContainer2 = new DockerContainer();
        disContainer2.setName("D3");
        disContainer2.setCommand("sh");
        disContainer2.setImage("test2");
        dis2.setDockerContainer(disContainer2);
        cds.add(dis2);

        String[] appnames = new String[]{"D1", "D2", "D3", "br:ovs"};
        int[][] topology = new int[][]{
                {0, 0, 0 ,0},
                {1, 0, 0 ,0},
                {0, 1, 0 ,0},
                {1, 1, 1 ,0}};

        clusterConfig.setDockers(cds);
        NetworkTopology topo = new NetworkTopology();
        topo.setAppNames(appnames);
        // 这个参数好像没啥用
        topo.setTopologyId(11);
        topo.setTopology(topology);
        clusterConfig.setTopology(topo);
        separatedClusterConfig.setClusterConfig(clusterConfig);
        clusterConfigs.add(separatedClusterConfig);

        clusterService.sendClusterConfigToEdgeNode(clusterConfigs);

        //下发逻辑拓扑
        clusterService.sendLogicTopoToEdgeNode(clusterConfigs);

        /*
        //改变逻辑拓扑
        int[][] newtopology = new int[][]{
                {0, 0, 0, 0 },
                {1, 0, 0, 0 },
                {1, 1, 0, 0 },
                {1, 1, 1, 0 }};
        topo.setTopology(newtopology);

        //下发修改后的逻辑拓扑
        clusterService.adjustLogicTopoToEdgeNode(clusterConfigs);

        */

    }

    //测试设置时间
    @Test
    public void test7() throws IllegalAccessException, InterruptedException, ParseException {
        ArrayList<SeparatedClusterConfig> clusterConfigs = new ArrayList<>();
        SeparatedClusterConfig separatedClusterConfig = new SeparatedClusterConfig();
        List<ContainerDescription> cds = new ArrayList<>();
        // TODO: 这个ID应该是从借口获取的
        separatedClusterConfig.setEdgeNodeId("192.168.190.135:18088");
        ClusterConfig clusterConfig = new ClusterConfig();
        clusterConfig.setId(11111l);
        clusterConfig.setOwner("joliu");

        //从文本中获取卫星的数据
        WeixingImpl wx = new WeixingImpl();
        ToponetImpl tp = new ToponetImpl();
        SimData sd = new SimData();
        //tp.delHistory();
        File file = new File("C:\\WorkSpace\\test\\topusim_1.txt");
        List<WeiXingData> wxData = wx.iniTopo(file,sd.getZhouqi());
        int[][] toponet = tp.getTopology(wxData,sd);


        //创建容器
        for (int k = 0; k < wxData.size(); k++) {
            ContainerDescription containerDescription = new ContainerDescription();
            containerDescription.setMode("normal");
            containerDescription.setReplicas(1);

            DockerContainer container = new DockerContainer();
            container.setName(wxData.get(k).getName());
            container.setCommand("sh");
            container.setImage("joliu/networktest");

            containerDescription.setDockerContainer(container);
            cds.add(containerDescription);
        }

        //设置创建时间
        //自己设计的时间
        //sd.setDefaultDate("2020-07-16 20-38-56");
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH-mm-ss");//24小时制
        clusterConfig.setCreateTime(sdf.parse(sd.getDefaultDate()));
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        clusterConfig.setDockers(cds);
        NetworkTopology topo = new NetworkTopology();
        topo.setAppNames(tp.getAppNames(wxData));
        // 这个参数好像没啥用
        topo.setTopologyId(11);
        topo.setTopology(toponet);
        clusterConfig.setTopology(topo);
        separatedClusterConfig.setClusterConfig(clusterConfig);
        clusterConfigs.add(separatedClusterConfig);

        clusterService.sendClusterConfigToEdgeNode(clusterConfigs);

        //下发逻辑拓扑
        clusterService.sendLogicTopoToEdgeNode(clusterConfigs);

        out:for (int i = 1; i<600; i++) {
            Thread.sleep(2000);
            int time = i * sd.getJiange();
            List<WeiXingData> changetp = wx.changeTopo(wxData,time);

            topo.setTopology(tp.getTopology(changetp,sd));

            //下发修改后逻辑拓扑
            clusterService.adjustLogicTopoToEdgeNode(clusterConfigs);
            Boolean tag = false;

            for(int k=1;k<toponet[0].length;k++){
                for (int j=0;j<k;j++){
                    if(toponet[k][j] == tp.getTopology(changetp, sd)[k][j]){
                        tag = true;
                    }
                }
            }
            if(tag){
                break out;
            }
        }

        System.out.println("所有topo完成");
    }
}
