package top.ninwoo;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;
import top.ninwoo.bishe.starter.entity.ContainerMonitor;
import top.ninwoo.bishe.starter.service.ClusterService;
import top.ninwoo.bishe.starter.service.NetworkService;
import top.ninwoo.common.entity.*;
import top.ninwoo.client.BisheTestMain;

import javax.annotation.Resource;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

@RunWith(SpringRunner.class)
@SpringBootTest(classes = BisheTestMain.class)
public class BisheTests {

    @Autowired
    private ClusterService clusterService;

    @Resource
    private NetworkService networkService;

    @Test
    public void testSendSeparateConfig() {
        ArrayList<SeparatedClusterConfig> clusterConfigs = new ArrayList<>();
        SeparatedClusterConfig separatedClusterConfig = new SeparatedClusterConfig();
        // TODO: 这个ID应该是从借口获取的
        separatedClusterConfig.setEdgeNodeId("192.168.190.135:18088");
        ClusterConfig clusterConfig = new ClusterConfig();
        clusterConfig.setId(11111l);
        clusterConfig.setOwner("joliu");
        ContainerDescription containerDescription = new ContainerDescription();
        containerDescription.setMode("normal");
        containerDescription.setReplicas(2);
        DockerContainer container = new DockerContainer();
        container.setName("Run");
        container.setCommand("sh");
        container.setImage("joliu/networktest");
        containerDescription.setDockerContainer(container);

        List<ContainerDescription> cds = new ArrayList<>();
        cds.add(containerDescription);

        ContainerDescription containerDescription1 = new ContainerDescription();
        containerDescription1.setMode("normal");
        containerDescription1.setReplicas(3);
        DockerContainer container1 = new DockerContainer();
        container1.setName("APP");
        container1.setCommand("sh");
        container1.setImage("joliu/networktest");
        containerDescription1.setDockerContainer(container1);

        cds.add(containerDescription1);

        clusterConfig.setDockers(cds);
        NetworkTopology topo = new NetworkTopology();
        topo.setAppNames(new String[]{"Run", "APP", "br:ovs1"});
        // 这个参数好像没啥用
        topo.setTopologyId(11);
        topo.setTopology(new int[][]{{0,0,0},{0,0,0},{1,1,0}});
        clusterConfig.setTopology(topo);
        separatedClusterConfig.setClusterConfig(clusterConfig);
        clusterConfigs.add(separatedClusterConfig);

        clusterService.sendClusterConfigToEdgeNode(clusterConfigs);
    }

    @Test
    public void removeClusterFromEdgeNodeTest() {
        clusterService.removeClusterFromEdgeNode(11111L);
    }

    @Test
    public void getLogicalNetworkTopologyTest() {
        NetworkTopology logicalNetworkTopology = networkService.getLogicalNetworkTopology(11111L);
        System.out.println(logicalNetworkTopology);
    }

    @Test
    public void getIpListByAppNameTest() {
        List<String> ipList = networkService.getIpListByAppName(11111L, "Run");
        System.out.println(ipList);
    }

    @Test
    public void enableNetworkMonitorTest() throws InterruptedException {
        String res = networkService.enableNetworkMonitor(11111L, "Run");
        System.out.println(res);
    }

    @Test
    public void cancelNetworkMonitorTest() {
        String run = networkService.cancelNetworkMonitor(11111L, "Run");
        System.out.println(run);
    }

    @Test
    public void getNetworkMonitorTest() throws InterruptedException {
        List<ContainerMonitor> run = networkService.getContainerMonitors(11111L, "Run");
        int i = 0;
        while(i < 10) {
            run.forEach(c -> {
                System.out.println(c.getContainerInfo());
            });
            Thread.sleep(500);
            i++;
        }
    }

    //改变集群拓扑配置
    private boolean adjustCluster(List<SeparatedClusterConfig> configs) {
        boolean b = clusterService.adjustClusterToEdgeNode(configs);
        return b;
    }

    //测试逻辑网路
    @Test
    public void test1() {
        ArrayList<SeparatedClusterConfig> clusterConfigs = new ArrayList<>();
        SeparatedClusterConfig separatedClusterConfig = new SeparatedClusterConfig();
        //需修改为下发的IP地址
        separatedClusterConfig.setEdgeNodeId("192.168.190.135:18088");

        ClusterConfig clusterConfig = new ClusterConfig();
        clusterConfig.setId(11111l);
        clusterConfig.setOwner("joliu");
        ContainerDescription containerDescription = new ContainerDescription();
        containerDescription.setMode("normal");
        containerDescription.setReplicas(1);
        DockerContainer container = new DockerContainer();
        container.setName("D1");
        container.setCommand("sh");
        container.setImage("test2");
        containerDescription.setDockerContainer(container);

        List<ContainerDescription> cds = new ArrayList<>();
        cds.add(containerDescription);

        ContainerDescription dis = new ContainerDescription();
        dis.setMode("normal");
        dis.setReplicas(1);
        DockerContainer disContainer = new DockerContainer();
        disContainer.setName("D2");
        disContainer.setCommand("sh");
        disContainer.setImage("test2");
        dis.setDockerContainer(disContainer);
        cds.add(dis);

        ContainerDescription dis2 = new ContainerDescription();
        dis2.setMode("normal");
        dis2.setReplicas(1);
        DockerContainer disContainer2 = new DockerContainer();
        disContainer2.setName("D3");
        disContainer2.setCommand("sh");
        disContainer2.setImage("test2");
        dis2.setDockerContainer(disContainer2);
        cds.add(dis2);

        String[] appnames = new String[]{"D1", "D2", "D3", "br:ovs"};
        int[][] topology = new int[][]{
                {0, 0, 0 ,0},
                {1, 0, 0 ,0},
                {0, 1, 0 ,0},
                {1, 1, 1 ,0}};

        clusterConfig.setDockers(cds);
        NetworkTopology topo = new NetworkTopology();
        topo.setAppNames(appnames);
        // 这个参数好像没啥用
        topo.setTopologyId(11);
        topo.setTopology(topology);
        clusterConfig.setTopology(topo);
        separatedClusterConfig.setClusterConfig(clusterConfig);
        clusterConfigs.add(separatedClusterConfig);

        clusterService.sendClusterConfigToEdgeNode(clusterConfigs);

        //下发逻辑拓扑
        clusterService.sendLogicTopoToEdgeNode(clusterConfigs);

        /*
        //改变逻辑拓扑
        int[][] newtopology = new int[][]{
                {0, 0, 0, 0 },
                {1, 0, 0, 0 },
                {1, 1, 0, 0 },
                {1, 1, 1, 0 }};
        topo.setTopology(newtopology);

        //下发修改后的逻辑拓扑
        clusterService.adjustLogicTopoToEdgeNode(clusterConfigs);

        */

    }

    //测试容器运行程序
    @Test
    public void test5() {
        ArrayList<SeparatedClusterConfig> clusterConfigs = new ArrayList<>();
        SeparatedClusterConfig separatedClusterConfig = new SeparatedClusterConfig();
        //需修改为下发的IP地址
        separatedClusterConfig.setEdgeNodeId("192.168.190.135:18088");

        ClusterConfig clusterConfig = new ClusterConfig();
        clusterConfig.setId(11111l);
        clusterConfig.setOwner("joliu");
        ContainerDescription containerDescription = new ContainerDescription();
        containerDescription.setMode("normal");
        containerDescription.setReplicas(1);
        DockerContainer container = new DockerContainer();
        container.setName("D1");
        container.setCommand("-e PARAMS=\"--name=" +container.getName()+"\"");
        container.setImage("joliu/networktest");
        containerDescription.setDockerContainer(container);

        List<ContainerDescription> cds = new ArrayList<>();
        cds.add(containerDescription);

        ContainerDescription dis = new ContainerDescription();
        dis.setMode("normal");
        dis.setReplicas(1);
        DockerContainer disContainer = new DockerContainer();
        disContainer.setName("D2");
        disContainer.setCommand("-e PARAMS=\"--name=" +disContainer.getName()+"\"");
        disContainer.setImage("joliu/networktest");
        dis.setDockerContainer(disContainer);
        cds.add(dis);

        ContainerDescription dis2 = new ContainerDescription();
        dis2.setMode("normal");
        dis2.setReplicas(1);
        DockerContainer disContainer2 = new DockerContainer();
        disContainer2.setName("D3");
        disContainer2.setCommand("-e PARAMS=\"--name=" +disContainer2.getName()+"\"");
        disContainer2.setImage("joliu/networktest");
        dis2.setDockerContainer(disContainer2);
        cds.add(dis2);

        String[] appnames = new String[]{"D1", "D2", "D3", "br:ovs"};
        int[][] topology = new int[][]{
                {0, 0, 0 ,0},
                {1, 0, 0 ,0},
                {0, 1, 0 ,0},
                {1, 1, 1 ,0}};

        clusterConfig.setDockers(cds);
        NetworkTopology topo = new NetworkTopology();
        topo.setAppNames(appnames);
        // 这个参数好像没啥用
        topo.setTopologyId(11);
        topo.setTopology(topology);
        clusterConfig.setTopology(topo);
        separatedClusterConfig.setClusterConfig(clusterConfig);
        clusterConfigs.add(separatedClusterConfig);

        clusterService.sendClusterConfigToEdgeNode(clusterConfigs);

        //下发逻辑拓扑
        clusterService.sendLogicTopoToEdgeNode(clusterConfigs);

        /*
        //改变逻辑拓扑
        int[][] newtopology = new int[][]{
                {0, 0, 0, 0 },
                {1, 0, 0, 0 },
                {1, 1, 0, 0 },
                {1, 1, 1, 0 }};
        topo.setTopology(newtopology);

        //下发修改后的逻辑拓扑
        clusterService.adjustLogicTopoToEdgeNode(clusterConfigs);

        */

    }
}
