package top.ninwoo;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;
import top.ninwoo.bishe.starter.service.ClusterService;
import top.ninwoo.bishe.starter.service.NetworkService;
import top.ninwoo.client.BisheTestMain;
import top.ninwoo.common.entity.*;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

@RunWith(SpringRunner.class)
@SpringBootTest(classes = BisheTestMain.class)
public class LunWenTests {
    @Autowired
    private ClusterService clusterService;

    @Resource
    private NetworkService networkService;

    @Test
    public void test1() {
        ArrayList<SeparatedClusterConfig> clusterConfigs = new ArrayList<>();
        SeparatedClusterConfig separatedClusterConfig = new SeparatedClusterConfig();
        separatedClusterConfig.setEdgeNodeId("192.168.190.135:8081");
        ClusterConfig clusterConfig = new ClusterConfig();
        clusterConfig.setId(11111l);
        clusterConfig.setOwner("joliu");
        ContainerDescription containerDescription = new ContainerDescription();
        containerDescription.setMode("normal");
        containerDescription.setReplicas(10);
        DockerContainer container = new DockerContainer();
        container.setName("D1");
        container.setCommand("sh");
        container.setImage("joliu/networktest");
        containerDescription.setDockerContainer(container);

        List<ContainerDescription> cds = new ArrayList<>();
        cds.add(containerDescription);

        ContainerDescription dis = new ContainerDescription();
        dis.setMode("normal");
        dis.setReplicas(1);
        DockerContainer disContainer = new DockerContainer();
        disContainer.setName("D2");
        disContainer.setCommand("sh");
        disContainer.setImage("joliu/networktest");
        dis.setDockerContainer(disContainer);
        cds.add(dis);

        clusterConfig.setDockers(cds);
        NetworkTopology topo = new NetworkTopology();
        topo.setAppNames(new String[]{"D1", "br:ovs1", "br:ovs2", "br:ovs3", "D2"});
        // 这个参数好像没啥用
        topo.setTopologyId(11);
        topo.setTopology(new int[][]{
                {0, 0, 0, 0, 0},
                {1, 0, 0, 0, 0},
                {0, 1, 0, 0, 0},
                {0, 0, 1, 0, 0},
                {0, 0, 0, 1, 0}});
        clusterConfig.setTopology(topo);
        separatedClusterConfig.setClusterConfig(clusterConfig);
        clusterConfigs.add(separatedClusterConfig);

        clusterService.sendClusterConfigToEdgeNode(clusterConfigs);
    }

    @Test
    public void test2() {
        ArrayList<SeparatedClusterConfig> clusterConfigs = new ArrayList<>();
        SeparatedClusterConfig separatedClusterConfig = new SeparatedClusterConfig();
        separatedClusterConfig.setEdgeNodeId("192.168.190.135:8081");
        ClusterConfig clusterConfig = new ClusterConfig();
        clusterConfig.setId(11111l);
        clusterConfig.setOwner("joliu");
        ContainerDescription containerDescription = new ContainerDescription();
        containerDescription.setMode("normal");
        containerDescription.setReplicas(5);
        DockerContainer container = new DockerContainer();
        container.setName("D1");
        container.setCommand("sh");
        container.setImage("joliu/networktest");
        containerDescription.setDockerContainer(container);

        List<ContainerDescription> cds = new ArrayList<>();
        cds.add(containerDescription);

        ContainerDescription dis = new ContainerDescription();
        dis.setMode("normal");
        dis.setReplicas(1);
        DockerContainer disContainer = new DockerContainer();
        disContainer.setName("D2");
        disContainer.setCommand("sh");
        disContainer.setImage("joliu/networktest");
        dis.setDockerContainer(disContainer);
        cds.add(dis);

        clusterConfig.setDockers(cds);
        NetworkTopology topo = new NetworkTopology();
        topo.setAppNames(new String[]{"D1", "br:ovs1", "br:ovs2", "br:ovs3", "D2"});
        // 这个参数好像没啥用
        topo.setTopologyId(11);
        topo.setTopology(new int[][]{
                {0, 0, 0, 0, 0},
                {1, 0, 0, 0, 0},
                {0, 1, 0, 0, 0},
                {0, 1, 1, 0, 0},
                {0, 0, 0, 1, 0}});
        clusterConfig.setTopology(topo);
        separatedClusterConfig.setClusterConfig(clusterConfig);
        clusterConfigs.add(separatedClusterConfig);

        clusterService.adjustClusterToEdgeNode(clusterConfigs);
    }

    @Test
    public void test3() throws InterruptedException {
        int i = 1;
        while(i < 40) {
            System.out.println(networkService.addQos(11111L, "D1", i + "mbit", "100ms"));
            i++;
            Thread.sleep(10000);
        }
    }

    @Test
    public void test4() throws InterruptedException {
        int i = 39;
        while(i > 0) {
            System.out.println(networkService.addQos(11111L, "D1", i + "mbit", "100ms"));
            i--;
            Thread.sleep(10000);
        }
    }

    @Test
    public void test5() {
        ArrayList<SeparatedClusterConfig> clusterConfigs = new ArrayList<>();
        SeparatedClusterConfig separatedClusterConfig = new SeparatedClusterConfig();
        separatedClusterConfig.setEdgeNodeId("127.0.0.1:8081");
        ClusterConfig clusterConfig = new ClusterConfig();
        clusterConfig.setId(11111l);
        clusterConfig.setOwner("joliu");
        ContainerDescription containerDescription = new ContainerDescription();
        containerDescription.setMode("normal");
        containerDescription.setReplicas(10);
        DockerContainer container = new DockerContainer();
        container.setName("NODE");
        container.setCommand("sh");
        container.setImage("joliu/networktest");
        containerDescription.setDockerContainer(container);

        List<ContainerDescription> cds = new ArrayList<>();
        cds.add(containerDescription);

        ContainerDescription dis = new ContainerDescription();
        dis.setMode("normal");
        dis.setReplicas(1);
        DockerContainer disContainer = new DockerContainer();
        disContainer.setName("DIS");
        disContainer.setCommand("sh");
        disContainer.setImage("joliu/networktest");
        dis.setDockerContainer(disContainer);
        cds.add(dis);

        ContainerDescription ftp = new ContainerDescription();
        ftp.setMode("normal");
        ftp.setReplicas(1);
        DockerContainer ftpContainer = new DockerContainer();
        ftpContainer.setName("FTP");
        ftpContainer.setCommand("sh");
        ftpContainer.setImage("joliu/networktest");
        ftp.setDockerContainer(ftpContainer);
        cds.add(ftp);

        ContainerDescription dis2 = new ContainerDescription();
        dis2.setMode("normal");
        dis2.setReplicas(1);
        DockerContainer dis2Container = new DockerContainer();
        dis2Container.setName("DISREC");
        dis2Container.setCommand("sh");
        dis2Container.setImage("joliu/networktest");
        dis2.setDockerContainer(dis2Container);
        cds.add(dis2);

        clusterConfig.setDockers(cds);
        NetworkTopology topo = new NetworkTopology();
        topo.setAppNames(new String[]{"FTP", "NODE", "DIS","DISREC", "br:ovs1", "br:ovs2", "br:ovs3"});
        // 这个参数好像没啥用
        topo.setTopologyId(11);
        topo.setTopology(new int[][]{
                {0,0,0,0,0,0,0},
                {0,0,0,0,0,0,0},
                {0,0,0,0,0,0,0},
                {0,0,0,0,0,0,0},
                {1,0,1,0,0,0,0},
                {0,1,0,0,1,0,0},
                {0,0,0,1,0,1,0}});
        clusterConfig.setTopology(topo);
        separatedClusterConfig.setClusterConfig(clusterConfig);
        clusterConfigs.add(separatedClusterConfig);

        clusterService.sendClusterConfigToEdgeNode(clusterConfigs);
    }

    @Test
    public void test6() {
        for (int i = 1; i <= 50; i++) {
            ArrayList<SeparatedClusterConfig> clusterConfigs = new ArrayList<>();
            SeparatedClusterConfig separatedClusterConfig = new SeparatedClusterConfig();
            separatedClusterConfig.setEdgeNodeId("127.0.0.1:8081");
            ClusterConfig clusterConfig = new ClusterConfig();
            clusterConfig.setId(11111l);
            clusterConfig.setOwner("joliu");
            ContainerDescription containerDescription = new ContainerDescription();
            containerDescription.setMode("normal");
            containerDescription.setReplicas(i);
            DockerContainer container = new DockerContainer();
            container.setName("D1");
            container.setCommand("sh");
            container.setImage("joliu/networktest");
            containerDescription.setDockerContainer(container);

            List<ContainerDescription> cds = new ArrayList<>();
            cds.add(containerDescription);

            clusterConfig.setDockers(cds);
            NetworkTopology topo = new NetworkTopology();
            topo.setAppNames(new String[]{"D1", "br:ovs1"});
            // 这个参数好像没啥用
            topo.setTopologyId(11);
            topo.setTopology(new int[][]{
                    {0, 0},
                    {1, 0}});
            clusterConfig.setTopology(topo);
            separatedClusterConfig.setClusterConfig(clusterConfig);
            clusterConfigs.add(separatedClusterConfig);
            long start = System.currentTimeMillis();
            clusterService.sendClusterConfigToEdgeNode(clusterConfigs);
            System.out.printf("%s,", System.currentTimeMillis() - start);
            clusterService.removeClusterFromEdgeNode(11111L);
        }

    }
}
